#include <npp.h> 
#include <iostream>

// Size of Image -- MxN where M = rows(height) , N = columns(width)

int main(int argc, char* argv[])
{
	int M = 8;
	int N = 8;

	cudaError_t err;

	// HostImage in CPU memory
	Npp8u * hostImage = (Npp8u *)malloc(sizeof(Npp8u)*M*N);

	for(int i=0;i < (M*N); ++i)
	hostImage[i] = (Npp8u)(rand()%10);
	
	printf("\nOriginal Image\n");
	printf("Size : %dx%d\n",M,N);
	for(int i=0;i< M; ++i)
	{	for(int j=0;j< N ; ++j) 
			printf("%d ", hostImage[i+ j *M]);
		printf("\n");
	}

	Npp8u * hostImageGPU;
	err = cudaMalloc( (void**) &hostImageGPU, sizeof(Npp8u)*M*N);	
	if(err != cudaSuccess)
	{	printf("Error in hostImage memory allocation\n");
		return -1;	
	}

    	err = cudaMemcpy( hostImageGPU, hostImage, sizeof(Npp8u)*M*N, cudaMemcpyHostToDevice);	
	if(err != cudaSuccess)
	{	printf("Error in hostImage memory copy\n");
		return -1;	
	}
 
       // create struct with box-filter mask size
        NppiSize oMaskSize = {3, 3};
        // set anchor point inside the mask to (0, 0)
        NppiPoint oAnchor = {0, 0};
	
	// ROI
        NppiSize oSizeROI = {N - oMaskSize.width + 1, M - oMaskSize.height + 1};

	// Allocate memory for DestImageGPU
	Npp8u * destImageGPU;
    	
	err = cudaMalloc( (void**) &destImageGPU, sizeof(Npp8u)*oSizeROI.width*oSizeROI.height);		
	if(err != cudaSuccess)
	{	printf("Error in destImage memory allocation\n");
		return -1;	
	}

	// Allocate memory for DestImage
	Npp8u * destImage = (Npp8u *)malloc( sizeof(Npp8u)*oSizeROI.width*oSizeROI.height);

        // run box filter -- hostImage Pitch = rows = M , destImage Pitch = oSizeROI.height
        NppStatus eStatusNPP;
        eStatusNPP = nppiFilterBox_8u_C1R(hostImageGPU, M, 
                                          destImageGPU, oSizeROI.height, 
                                          oSizeROI, oMaskSize, oAnchor);

        if(NPP_NO_ERROR != eStatusNPP);
 	{	
		printf("Error in NPP call\n");
		printf("NPP error status : %d\n", eStatusNPP);
		return -1;
	}

	// Copy filtered image
    	cudaMemcpy( destImage, destImageGPU , sizeof(Npp8u)*oSizeROI.width*oSizeROI.height, cudaMemcpyDeviceToHost);
	
	printf("\nFiltered Image\n");
	printf("Size: %dx%d\n",oSizeROI.height, oSizeROI.width );

	for(int i=0 ; i < oSizeROI.height ; ++i)
	{	for(int j=0 ; j<  oSizeROI.width ; ++j) 
			printf("%d ", destImage[i+ j *M]);
		printf("\n");
	}

	// Cleanup memory
	free(hostImage);
	free(destImage);
    	cudaFree(hostImageGPU);
    	cudaFree(destImageGPU);

   	cudaThreadExit();  
}

